<?php

class SearchEngine{
	private $options;
	private $uselessWords;
	public function __construct(){
		$this->options=array("application"=>true,"useCase"=>true,"systemMessage"=>true,"userAction"=>true,"description"=>true,"expressionComp"=>1/2,"wordComp"=>1);
		$this->uselessWords=array("sur","le","la","les","l","un","une","de","des","d","et","m","n","c","mon","ma","mes","ton","ta","tes","t","son","sa","ses","s","ces","ce","cet","cette");
	}
	
	/**
	 * Nettoyage d'un mot avant comparaison  un autre
	 * Suppression des doubles lettres conscutives, des pluriels, des traits d'union
	 * @param string $aWord
	 */
	public function cleanWord($aWord){
		$aWord = htmlentities($aWord, ENT_NOQUOTES,'utf-8');
		
		$aWord = preg_replace('#&([A-za-z])(?:acute|cedil|circ|grave|orn|ring|slash|th|tilde|uml);#', '\1', $aWord); //supprime les accents
		$aWord = preg_replace('#&([A-za-z]{2})(?:lig);#', '\1', $aWord); // pour les ligatures e.g. '&oelig;'
		$aWord = preg_replace('#&[^;]+;#', '', $aWord); // supprime les autres caractres
		
		
		$aWord = strtolower($aWord); // Majuscule => minuscule
		
		$aWord=preg_replace("/(\w{2,})s/u","$1",$aWord); // Pluriels => singulier
		$aWord=preg_replace("<(.)\\1+(?!\\1)>", "$1", $aWord); // Double lettre => une lettre
		
		return $aWord;
	}
	
	/**
	 * Suppresion des mots inutiles
	 * @param string $anExpression
	 */
	public function cleanExpression($anExpression){
		
		$anExpression=preg_replace("/\s|[\'\-\,\.\;\?\!\:]/", "#espace#", $anExpression);
		$anExpression=preg_replace("<(#espace#)\\1+(?!\\1)>", "$1", $anExpression); // Double #espace# => un #espace#
		$words=explode("#espace#", $anExpression);
		
		$wordsCorr=array();
		foreach ($words as $word){
			$word=$this->cleanWord($word);
			$wordsCorr[]=$word;
		}
		$words=array_diff($wordsCorr, $this->uselessWords);
		return implode(" ", $words);
	}
	
	/**
	 * Compare deux mots
	 * @param string $word1
	 * @param string $word2
	 * @return boolean vrai si les deux mots sont identiques
	 */
	public function wordCompare($word1,$word2){
		$mot1=$this->cleanWord($word1);
		$mot2=$this->cleanWord($word2);
		similar_text($word1, $word2,$percent);
		return number_format($percent,0)>=$this->options["wordComp"];
	}
	
	/**
	 * Vrifie si un mot est compris dans une expression
	 * @param string $aWord
	 * @param string $anExpression
	 * @return boolean
	 */
	public function containsWord($aWord,$anExpression){
		$anExpression=$this->cleanExpression($anExpression);
		$words=explode(" ", $anExpression);
		$trouve=false;
		$count=count($words);
		$i=0;
		while($trouve!==TRUE && $i<$count){
			$trouve=$this->wordCompare($aWord, $words[$i]);
			$i++;
		}
		return $trouve;
	}
	
	/**
	 * Compare deux expressions
	 * @param string $expression1
	 * @param string $expression2
	 * @return boolean vrai si les deux expressions sont identiques
	 */
	private function expressionCompare($expression1,$expression2){
		$expression1=$this->cleanExpression($expression1);
		$expression2=$this->cleanExpression($expression2);
		$words=explode(" ", $expression1);
		$count=count($words);
		$countEquals=0;
		$equals=false;
		$i=0;
		while($equals!==TRUE && $i<$count){
			if($this->containsWord($words[$i], $expression2))
				$countEquals++;
			$equals=(($countEquals/$count)>=$this->options["expressionComp"]);
			$i++;
		}
		return $equals;
	}
	
	/**
	 * Compare deux reports
	 * @param Report $report1
	 * @param Report $report2
	 * @return boolean
	 */
	public function compare(Report $report1, Report $report2){		
		$ok =0;
		if(($this->options["useCase"])&&($report1->getCode()->getCode()!=$report2->getCode()->getCode())){
			return false;
		}
		
		if(($this->options["systemMessage"])&&(!$this->expressionCompare($report1->getSystemmessage(),$report2->getSystemmessage()))){
			$ok=$ok+1;
	
		}
		
		if(($this->options["userAction"])&&(!$this->expressionCompare($report1->getUseraction(),$report2->getUseraction()))){
			$ok=$ok+1;
		}
		
		if(($this->options["description"])&&(!$this->expressionCompare($report1->getDescriptif(),$report2->getDescriptif()))){
			$ok=$ok+1;
		}
		if($ok>1)
		{
			return false;
		}
		return true;
	}
	
	public function addUselessWord($word){
	} 
	
	/**
	 * Modifie une option
	 * @param string $key
	 * @param unknown $value
	 */
	public function setOption($key,$value){
		$this->options[$key]=$value;
	}
}